package de.hftstuttgart.dtabackend.utils;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class RegexUtil {

	public final static String DTA_TESTCONFIGREGEX = "^dta::(.*)::(.*|none)::(.*|none)::(.*)::(.*)$";
	public final static String DTA_SUBMISSIONCONFIGREGEX = "^dta::(.*)::(.*|none)::(.*|none)::(.*)$";
	public final static String TESTCONFIGREGEX = "^dtt::(.*)::(.*|none)::(.*|none)::(.*)$";
	public final static String SUBMISSIONCONFIGREGEX = "^dtt::(.*)::(.*|none)::(.*|none)$";
	
    public enum ConfigType {
        TEACHER,
        STUDENT,
    }

    private static final Logger LOG = LogManager.getLogger(RegexUtil.class);

    public static Matcher findStudentConfig(InputStream is) {
        return findConfig(is, ConfigType.STUDENT);
    }

    public static Matcher findProfessorConfig(InputStream is) {
        return findConfig(is, ConfigType.TEACHER);
    }

    public static Matcher findConfig(InputStream is, ConfigType configType) {
        Pattern pattern;
        switch (configType) {
            case TEACHER:
                pattern = Pattern.compile(RegexUtil.TESTCONFIGREGEX);
                break;

            case STUDENT:
                pattern = Pattern.compile(RegexUtil.SUBMISSIONCONFIGREGEX);
                break;

            default:
                String msg = String.format("unknown config type: %s", configType.name());
                LOG.error(msg);
                throw new RuntimeException(msg);
        }

        Matcher config = extractConfig(is, pattern);

        return config;
    }
    
	public static Matcher extractConfig(InputStream configFileStream, Pattern pattern) {
		LOG.debug("reading configuration file");
		Matcher configItems=null;
		// open saved config in a try-with
        try (BufferedReader br = new BufferedReader(new InputStreamReader(configFileStream))) {
            String line;

            // search for a URI while none is found and there are lines left
            while (configItems == null && (line = br.readLine()) != null) {
                Matcher matcher = pattern.matcher(line);
                if (matcher.matches()) {
                    LOG.debug(String.format("found valid config line: %s", line));
                    configItems = matcher;
                }
            }
        } catch (IOException e) {
            LOG.error("Error while reading repo config", e);
        }
        finally {
        }
		return configItems;
	}
}
